!=============================================================================================
! EXAMPLE DRIVER PROGRAM FOR STWHAM / PTWHAM IMPLEMENTATION.
!
! Modify parameters K, L, N, filenames, and target_temperature for use with your own data,
! or add command-line options.
!=============================================================================================
! Written by John D. Chodera <jchodera@gmail.com>, Dill lab, UCSF, 2005-2006.
!
! Copyright (c) 2006 The Regents of the University of California.  All Rights Reserved.
!
! This program is free software; you can redistribute it and/or modify it under the terms of
! the GNU General Public License as published by the Free Software Foundation; either version 2
! of the License, or (at your option) any later version.
! 
! This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
! without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
! See the GNU General Public License for more details.
! 
! You should have received a copy of the GNU General Public License along with this program;
! if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
! Boston, MA  02110-1301, USA.
!=============================================================================================
program PTWHAM_driver
  use numeric_kinds
  use PTWHAM

  implicit none
  
  ! Parameters
  integer, parameter :: K = 10   ! number of replicas
  integer, parameter :: L = 10   ! number of temperatures
  integer, parameter :: N = 10000 ! number of snapshots/replica
  real(dp), parameter :: kB = 1.381 * 6.02214 / 4184.0 ! Boltzmann's constant, in kcal/(mol K)
  real(dp), parameter :: target_temperature = 300.0     ! target temperature, in K
  real(dp), parameter :: beta = 1.0 / (kB * target_temperature)
  character(len=*), parameter :: betaFilename = '../testdata/by-replica/beta.dat' 
      ! name of file containing inverse temperatures
  character(len=*), parameter :: energyFilename = '../testdata/by-replica/energies.dat' 
      ! name of file containing energies -- each replica is a column, each snapshot a row
  character(len=*), parameter :: tempIndexFilename = '../testdata/by-replica/temp_index.dat' 
      ! name of file containing indices of temperatures visited by each replica
  character(len=*), parameter :: observableFilename = '../testdata/by-replica/end-to-end-dist.dat' 
      ! name of file containing observable -- each replica is a column, each snapshot a row
  character(len=*), parameter :: stateIndexFilename = '../testdata/by-replica/psi_state.dat' 
      ! name of file containing states
  integer, parameter :: nstates = 36 ! number of states for PMF
  integer, parameter :: fileUnit = 2 ! unit to use for file access

  ! Local variables
  real(dp), dimension(:), allocatable      :: beta_l         ! inverse temperatures
  real(dp), dimension(:,:), allocatable    :: U_kn           ! potential energies
  integer, dimension(:,:), allocatable :: temp_index_kn  ! temperature indices
  integer, dimension(:,:), allocatable :: state_index_kn ! state indices for PMF
  real(dp), dimension(:,:), allocatable    :: A_kn 
     ! A_kn(k,n) is the observable A for snapshot n of replica k
  real(dp) :: A, dA ! estimate of expectation and uncertainty of an observable
  real(dp), dimension(:), allocatable      :: F_i, dF_i 
     ! potential of mean force and associated uncertainty
  integer :: iIndex ! state index

  !===========================================================================================
  ! Read simulation data and initialize PTWHAM.
  !===========================================================================================

  ! Read inverse temperatures.
  write(*,*) 'Reading inverse temperatures from ', betaFilename, '...'
  allocate(beta_l(L))
  open(unit=fileUnit, file=betaFilename, status='old', access='sequential', form='formatted')
  read(unit=fileUnit, fmt=*) beta_l
  close(unit=fileUnit)
  
  ! Read snapshot energies.
  write(*,*) 'Reading energies from ', energyFilename, '...'
  allocate(U_kn(K,N))
  open(unit=fileUnit, file=energyFilename, status='old', access='sequential', form='formatted')
  read(unit=fileUnit, fmt=*) U_kn
  close(unit=fileUnit)

  ! Read temperature indices.
  write(*,*) 'Reading temperature indices from ', tempIndexFilename, '...'
  allocate(temp_index_kn(K,N))
  open(unit=fileUnit, file=tempIndexFilename, status='old', access='sequential', form='formatted')
  read(unit=fileUnit, fmt=*) temp_index_kn
  close(unit=fileUnit)

  ! Read state indices for the calculation of a PMF.
  write(*,*) 'Reading state indices from ', stateIndexFilename, '...'
  allocate(state_index_kn(K,N))
  open(unit=fileUnit, file=stateIndexFilename, status='old', access='sequential', form='formatted')
  read(unit=fileUnit, fmt=*) state_index_kn
  close(unit=fileUnit)

  ! Construct the histogram bins and counts.
  write(*,*) 'Constructing histogram counts...'
  call PTWHAM_initialize(U_kn, beta_l, temp_index_kn)

  !===========================================================================================
  ! Estimate the expectation and uncertainty for an observable.
  !===========================================================================================

  ! Read observable.
  write(*,*) 'Reading observable from ', observableFilename, '...'
  allocate(A_kn(K,N))
  open(unit=fileUnit, file=observableFilename, status='old', access='sequential', form='formatted')
  read(unit=fileUnit, fmt=*) A_kn
  close(unit=fileUnit)

  ! Compute expectation and uncertainty of an observable.
  write(*,*) 'Computing expectation and uncertainty of an observable...'
  call PTWHAM_computeExpectation(beta, A_kn, A, dA)
  write(*,*) 'A = ', A, ', dA = ', dA

  !===========================================================================================
  ! Compute a potential of mean force.
  !===========================================================================================

  ! Compute potential of mean force.
  write(*,*) 'Computing potential of mean force...'
  allocate(F_i(nstates),dF_i(nstates))
  call PTWHAM_computePMF(beta, state_index_kn, nstates, F_i, dF_i)
  write(*,'(a8,a8,a8)') 'bin', 'F_i', 'dF_i'
  do iIndex = 1,nstates
     write(*,'(i8,f8.4,f8.4)') iIndex, F_i(iIndex), dF_i(iIndex)
  end do

  !===========================================================================================
  ! Clean up.
  !===========================================================================================
  write(*,*) 'Cleaning up...'
  call PTWHAM_finalize()
  write(*,*) 'Done.'

end program PTWHAM_driver
